"""
Core FleetFluid implementation with PydanticAI agents.
"""

import asyncio
from typing import Any, Dict, List, Optional, Union
from pydantic import BaseModel, Field
from pydantic_ai import Agent
from pydantic_ai.models import KnownModelName


class SingleLabelResult(BaseModel):
    """Result for single label classification."""
    label: str = Field(description="The selected label from the provided options")
    confidence: float = Field(description="Confidence score between 0 and 1", ge=0, le=1)
    reasoning: str = Field(description="Brief explanation of why this label was chosen")


class MultipleLabelResult(BaseModel):
    """Result for multiple label classification."""
    labels: List[str] = Field(description="List of selected labels from the provided options")
    confidence_scores: Dict[str, float] = Field(description="Confidence score for each selected label")
    reasoning: str = Field(description="Brief explanation of why these labels were chosen")


class FleetFluid:
    """Main FleetFluid class containing AI agent functions."""
    
    def __init__(self, model: str = "openai:gpt-4", **kwargs):
        """
        Initialize FleetFluid with model configuration.
        
        Args:
            model: Model identifier for PydanticAI
            **kwargs: Additional configuration for PydanticAI agents
        """
        self.model = model
        self.agent_config = kwargs
        self._ai_agent: Optional[Agent] = None
        self._label_agent: Optional[Agent] = None
    
    def _get_ai_agent(self) -> Agent:
        """Lazy initialization of the AI transformation agent."""
        if self._ai_agent is None:
            self._ai_agent = Agent(
                model=self.model,
                system_prompt=(
                    "You are a helpful AI assistant specialized in text transformation and processing. "
                    "You will receive a transformation prompt and input data. "
                    "Apply the requested transformation accurately and return only the transformed result. "
                    "Do not include explanations, markdown formatting, or additional text unless specifically requested."
                ),
                **self.agent_config
            )
        return self._ai_agent
    
    def _get_label_agent(self) -> Agent:
        """Lazy initialization of the labeling agent."""
        if self._label_agent is None:
            self._label_agent = Agent(
                model=self.model,
                system_prompt=(
                    "You are a specialized AI assistant for text classification and labeling. "
                    "You will receive text content and a list of possible labels. "
                    "Your task is to select the most appropriate label(s) based on the content. "
                    "Be precise and thoughtful in your selection, considering the context and meaning of the text."
                ),
                **self.agent_config
            )
        return self._label_agent
    
    def ai(self, prompt: str, data: str) -> str:
        """
        Apply AI transformation to data using the given prompt.
        
        This is a synchronous wrapper around the async PydanticAI agent.
        
        Args:
            prompt: Instruction for the AI (e.g., "write it grammatically correct", "translate to Spanish")
            data: Input data to transform
            
        Returns:
            Transformed data as string
        """
        return asyncio.run(self._ai_async(prompt, data))
    
    def label(self, text: str, labels: List[str], multiple: bool = False) -> Union[SingleLabelResult, MultipleLabelResult]:
        """
        Label text using AI agent with structured output.
        
        Args:
            text: Input text to label
            labels: List of possible labels to choose from
            multiple: If True, select multiple labels; if False, select single best label
            
        Returns:
            Structured result with selected label(s) and confidence scores
        """
        return asyncio.run(self._label_async(text, labels, multiple))
    
    async def _ai_async(self, prompt: str, data: str) -> str:
        """Async implementation of AI transformation."""
        agent = self._get_ai_agent()
        
        # Combine the transformation prompt with the data
        full_prompt = f"{prompt}\n\nInput data: {data}"
        
        try:
            result = await agent.run(full_prompt)
            return str(result.data)
        except Exception as e:
            raise RuntimeError(f"AI transformation failed: {str(e)}") from e
    
    async def _label_async(self, text: str, labels: List[str], multiple: bool = False) -> Union[SingleLabelResult, MultipleLabelResult]:
        """Async implementation of labeling."""
        agent = self._get_label_agent()
        
        # Create the prompt for labeling
        labels_str = ", ".join(labels)
        if multiple:
            prompt = f"""
            Analyze the following text and select the most appropriate labels from the provided options.
            
            Text: {text}
            Available labels: {labels_str}
            
            Select multiple labels that best describe the text content. You can choose one or more labels as appropriate.
            """
            result_class = MultipleLabelResult
        else:
            prompt = f"""
            Analyze the following text and select the single most appropriate label from the provided options.
            
            Text: {text}
            Available labels: {labels_str}
            
            Select the single best label that most accurately describes the text content.
            """
            result_class = SingleLabelResult
        
        try:
            # Use the correct PydanticAI structured output approach
            result = await agent.run(prompt, output_type=result_class)
            return result.data
        except Exception as e:
            raise RuntimeError(f"Labeling failed: {str(e)}") from e
    
    async def ai_async(self, prompt: str, data: str) -> str:
        """
        Async version of ai() method for use in async contexts.
        
        Args:
            prompt: Instruction for the AI
            data: Input data to transform
            
        Returns:
            Transformed data as string
        """
        return await self._ai_async(prompt, data)
    
    async def label_async(self, text: str, labels: List[str], multiple: bool = False) -> Union[SingleLabelResult, MultipleLabelResult]:
        """
        Async version of label() method for use in async contexts.
        
        Args:
            text: Input text to label
            labels: List of possible labels to choose from
            multiple: If True, select multiple labels; if False, select single best label
            
        Returns:
            Structured result with selected label(s) and confidence scores
        """
        return await self._label_async(text, labels, multiple)
    